<?php
/**
 * @package   OSEmbed-Pro
 * @contact   www.joomlashack.com, help@joomlashack.com
 * @copyright 2016-2022 Joomlashack.com. All rights reserved
 * @license   http://www.gnu.org/licenses/gpl.html GNU/GPL
 *
 * This file is part of OSEmbed-Pro.
 *
 * OSEmbed-Pro is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * OSEmbed-Pro is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OSEmbed-Pro.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Alledia\OSEmbed\Pro;

use Alledia\Framework\Factory;
use Alledia\OSEmbed\Free\Helper as HelperFree;
use Exception;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Log\Log;
use Joomla\Registry\Registry;
use WFEditor;

defined('_JEXEC') or die();

abstract class Helper extends HelperFree
{
    /**
     * @param array $expectedContexts
     *
     * @return bool
     * @throws Exception
     */
    public static function checkContext(array $expectedContexts)
    {
        $app = Factory::getApplication();

        foreach ($expectedContexts as $context) {
            $match = 0;

            foreach ($context as $input => $expected) {
                if ($app->input->getCmd($input) === $expected) {
                    $match += 1;
                }
            }

            if ($match === count($context)) {
                return true;
            }
        }

        return false;
    }

    /**
     * @return array
     */
    protected static function getEditorData()
    {
        $user              = Factory::getUser();
        $params            = new Registry($user->params);
        $editor            = $params->get('editor');
        $usingGlobalEditor = false;

        if (empty($editor)) {
            // Check the editor set in the global configuration
            $config            = Factory::getConfig();
            $editor            = $config->get('editor');
            $usingGlobalEditor = true;
        }

        return [
            'name'   => $editor,
            'global' => $usingGlobalEditor,
            'user'   => $user
        ];
    }

    /**
     * @return bool
     */
    public static function checkRequiredEditor()
    {
        $supportedEditors = ['tinymce', 'jce'];

        // Check if the user has a custom editor
        $editorData = static::getEditorData();

        if (!in_array($editorData['name'], $supportedEditors)) {
            $log = 'OSEmbed requires one of the following editors: '
                . implode(', ', $supportedEditors) . '. ';
            if (!$editorData['global']) {
                $log .= 'The user ' . $editorData['user'] . ' is using ' . $editorData['name'] . '. '
                    . 'OSEmbed preview will not work for it';
            } else {
                $log = 'The global editor setting is set to ' . $editorData['name'] . '. '
                    . 'OSEmbed preview will not work for users with the default editor';
            }

            Log::add($log, Log::WARNING, 'osembed.library');

            return false;
        }

        return true;
    }

    /**
     * @param string $editorName
     *
     * @return bool
     */
    public static function checkEditorIsInstalled($editorName)
    {
        $editor = Factory::getExtension($editorName, 'plugin', 'editors');
        $id     = $editor->getId();

        return !empty($id);
    }

    /**
     * @param array $editorData
     * @param bool  $addLog
     *
     * @return bool
     * @throws Exception
     */
    public static function checkRequiredEditorSettings($editorData = null, $addLog = true)
    {
        $app = Factory::getApplication();

        if (empty($editorData)) {
            $editorData = static::getEditorData();
        }

        if ($editorData['name'] === 'tinymce') {
            $editorPlugin    = Factory::getExtension('tinymce', 'plugin', 'editors');
            $params          = $editorPlugin->params;
            $invalidElements = explode(',', $params->get('invalid_elements', ''));
            $invalidElements = array_map('trim', $invalidElements);

            if (in_array('iframe', $invalidElements)) {
                if ($addLog) {
                    Log::add(
                        Text::_('PLG_SYSTEM_OSEMBEDPREVIEW_TINYMCE_INVALID_SETTINGS_LOG'),
                        Log::WARNING,
                        'osembed.library'
                    );
                    $app->enqueueMessage(Text::_('PLG_SYSTEM_OSEMBEDPREVIEW_TINYMCE_INVALID_SETTINGS'), 'warning');
                }

                return false;
            }

        } elseif ($editorData['name'] === 'jce') {
            if (!class_exists('WFEditor')) {
                require JPATH_ADMINISTRATOR . '/components/com_jce/includes/base.php';
                wfimport('editor.libraries.classes.editor');
            }

            /** @var WFEditor $wf */
            $wf = WFEditor::getInstance();

            $extendedElements = explode(
                ',',
                preg_replace('#\s+#', '', $wf->getParam('editor.extended_elements', '', ''))
            );

            $valid = false;
            // Iframe shoud be an extended element, alone or with defined attributes: iframe, iframe[src|width]
            if (!empty($extendedElements)) {
                foreach ($extendedElements as $elem) {
                    if (substr_count($elem, 'iframe') > 0) {
                        $valid = true;
                        break;
                    }
                }
            }

            if (!$valid) {
                if ($addLog) {
                    Log::add(
                        Text::_('PLG_SYSTEM_OSEMBEDPREVIEW_JCE_INVALID_SETTINGS_LOG'),
                        Log::WARNING,
                        'osembed.library'
                    );
                    $app->enqueueMessage(Text::_('PLG_SYSTEM_OSEMBEDPREVIEW_JCE_INVALID_SETTINGS'), 'warning');
                }
            }

            return $valid;
        }

        return true;
    }

    /**
     * @return bool
     * @throws Exception
     */
    public static function complyBasicRequirementsForPreview()
    {
        // @todo: Add tests for requirements here, like: iframe tag.
        // Show warnings if the user has permission to change the required settings into the editor

        $complies = static::checkRequiredEditor();

        if ($complies) {
            $complies = static::checkRequiredEditorSettings();
        }

        return $complies;
    }

    public static function requestedFromSameDomain()
    {
        $input          = Factory::getApplication()->input;
        $myDomain       = $input->server->getString('SCRIPT_URI');
        $requestsSource = $input->server->get('HTTP_REFERER');

        // Failback in case Apache/nginx do not provide the required information
        if (empty($myDomain) || empty($requestsSource)) {
            return true;
        }

        return parse_url($myDomain, PHP_URL_HOST) === parse_url($requestsSource, PHP_URL_HOST);
    }

    /**
     * @return bool
     * @throws Exception
     */
    public static function detectParseHook()
    {
        $contexts = [['plg_task' => 'osembedpreview.parse_content']];

        return Helper::checkContext($contexts);
    }
}
