<?php
/**
 * @package   OSEmbed-Pro
 * @contact   www.joomlashack.com, help@joomlashack.com
 * @copyright 2016-2022 Joomlashack.com. All rights reserved
 * @license   http://www.gnu.org/licenses/gpl.html GNU/GPL
 *
 * This file is part of OSEmbed-Pro.
 *
 * OSEmbed-Pro is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * OSEmbed-Pro is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OSEmbed-Pro.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Alledia\OSEmbed\Pro\Provider;

defined('_JEXEC') or die();

use Embera\Provider\ProviderAdapter;
use Embera\Provider\ProviderInterface;
use Embera\Url;
use Joomla\Utilities\ArrayHelper;

class GoogleDocs extends ProviderAdapter implements ProviderInterface
{
    /**
     * @inheritdoc
     */
    protected $endpoint = 'https://docs.google.com';

    /**
     * @inheritdoc
     */
    protected static $hosts = [
        'docs.google.com',
    ];

    /**
     * @inheritdoc
     */
    protected $httpsSupport = true;

    /**
     * @inheritdoc
     */
    protected $allowedParams = ['maxwidth', 'maxheight', 'preview'];

    /**
     * @inheritdoc
     */
    public function validateUrl(Url $url)
    {
        $regex = '~http[s]?:\/\/((?:www\.)?docs\.google\.com\/(?:document|presentation|spreadsheets|forms|drawings)\/[a-z0-9\/\?=_\-\.\,&%\$#\@\!\+]*)~i';

        return preg_match($regex, $url);
    }

    /**
     * Google docs does not support oembed.
     * This method fakes an oembed response.
     *
     * @return array with data that the oembed response should have
     */
    public function getFakeResponse()
    {
        $params    = $this->getParams();
        $preview   = !empty($params['preview']);
        $iframeSrc = html_entity_decode($this->url);
        $parts     = explode('?', $iframeSrc);

        $iframeSrc = array_shift($parts);
        parse_str(array_shift($parts), $iframeQuery);

        if ($preview) {
            $iframeSrc = str_ireplace('/edit', '', $iframeSrc);
            $iframeSrc .= '/preview';
            if (isset($iframeQuery['usp'])) {
                unset($iframeQuery['usp']);
            }
        }

        // Get the type of document
        preg_match('~google\.com/(document|presentation|spreadsheets|forms|drawings)/~i', $iframeSrc, $matches);
        $type = $matches[1];

        switch ($type) {
            case 'document':
                // Strip /preview if using Publish link
                if (preg_match('~/pub/preview$~i', $iframeSrc)) {
                    $iframeSrc = str_replace('/pub/preview', '/pub', $iframeSrc);
                }
                break;

            case 'presentation':
                // If needed, convert /pub to /embed and strip /preview
                if (preg_match('~/pub(?:/preview)?$~i', $iframeSrc, $match)) {
                    $iframeSrc = str_replace($match[0], '/embed', $iframeSrc);
                }
                break;

            case 'spreadsheets':
                // Strip /preview if using published spreadsheet
                if (preg_match('~/pubhtml(?:/preview)?$~i', $iframeSrc)) {
                    // Published version of spreadsheet
                    $iframeSrc = str_ireplace('/pubhtml/preview', '/pubhtml', $iframeSrc);
                }
                break;

            case 'forms':
                $iframeSrc = preg_replace('~(/edit|/preview)$~', '', $iframeSrc);
                if (stripos($iframeSrc, '/viewform') === false) {
                    $iframeSrc .= '/viewform';
                }
                break;

            case 'drawings':
                if (preg_match('~/pub(?:/preview)?$~i', $iframeSrc, $match)) {
                    $src = str_ireplace($match[0], '/pub', $iframeSrc);
                    if ($iframeQuery) {
                        $src .= '?' . http_build_query($iframeQuery);
                    }
                    $attributes = [
                        'src' => $src
                    ];
                    if (isset($iframeQuery['w'])) {
                        $attributes['width'] = $iframeQuery['w'];
                    }
                    if (isset($iframeQuery['h'])) {
                        $attributes['height'] = $iframeQuery['h'];
                    }


                    $html = sprintf('<img %s>', ArrayHelper::toString($attributes));
                }
                break;
        }

        if (empty($html)) {
            if ($iframeQuery) {
                $iframeSrc .= '?' . http_build_query($iframeQuery);
            }

            $attributes = [
                'src'                   => $iframeSrc,
                'frameborder'           => '0',
                'width'                 => '600',
                'height'                => '450',
                'allowfullscreen'       => 'true',
                'mozallowfullscreen'    => 'true',
                'webkitallowfullscreen' => 'true'
            ];

            $html = sprintf('<iframe %s></iframe>', ArrayHelper::toString($attributes));
        }

        return [
            'type'          => $this->getProviderName() . '-' . $type,
            'provider_name' => 'Google Docs',
            'provider_url'  => $this->endpoint,
            'title'         => 'Unknown title',
            'html'          => $html,
        ];
    }
}
